'use strict';

describe('hljs', function() {
  require('./autoDetection');
  require('./beginKeywords');
  require('./binaryNumber');
  require('./cNumber');
  require('./fixmarkup');
  require('./getLanguage');
  require('./highlight');
  require('./ident');
  require('./keywords');
  require('./number');
  require('./registerAlias');
  require('./unregisterLanguage');
  require('./starters');
  require('./underscoreIdent');
});
